% fmrlab_chkdataset() - check the necessary fields in FMRI data
%                       structure in FMRLAB, make FMRLAB back-
%                       compatible with the old datasets
%
% Usage:
%   >> fmrlab_chkdataset;
%
% Inputs:
%
% Outputs:
%
% Author: Jeng-Ren Duann, SCCN/INC/UCSD & CNL/Salk Inst., 2003-09-24
%
% See also: set_struc_info(), modify_param(), save_dataset(),
%           set_fmri_global_ui()
%
% Notes:
%    - necessary fields in FMRI data structural
% 	FMRI.data	image body
%	FMRI.filename	image file name
%	FMRI.pathname	image path
%	FMRI.dime_x	image width (in pixels)
%	FMRI.dime_y	image height (in pixels)
%	FMRI.dime_z	image depth (in pixels)
%	FMRI.fov_x	field of view in x direction (in mm)
%	FMRI.fov_y	field of view in y direction (in mm)
%	FMRI.slicetk	slice thickness (in mm)
%	FMRI.dummy	dummy scans conducted in fMRI experiment
%	FMRI.timepoint	real time point acquired in fMRI experiment
%			total time point = dummy + timepoint
%	FMRI.tr		inter-scan interval = TR (in sec)
%       FMRI.idx        index of brain voxels
% 	FMRI.processed	recording the stage of image processing
%			it can be: 'raw', 'slice_timed',
%			'deskulled', 'icaed'
%       FMRI.alignslice reference slice of image for slice-timing process
%       FMRI.struc_name filename of structural images
%       FMRI.struc_path path name of structural images
%       FMRI.struc_image_width image width of structural images
%       FMRI.struc_image_height image height of structural images
%       FMRI.struc_image_depth  slice_number of structural images
%       FMRI.struc_fov_x field of view in x direction of structural images
%       FMRI.struc_fov_y field of view in y direction of structural images  
%       FMRI.struc_slicetk slice thickness of structural images
%       FMRI.struc_resort resort image or not (1: resort; 0: no resort)
%       FMRI.struc_flip  flip image or not (1: flip; 0: no flip)
%
%    - ICA VARIABLE:
%	FMRI.pcano	dimensionality to be reduced to (default = 0, all)
%	FMRI.annealdeg	annleaing angle used in ICA training
%	                (default = 60 degree)
%	FMRI.extended	if extended ICA used (default = 0, no)
%       FMRI.maxsteps   maximum steps for ICA training
%	FMRI.icasphere	sphere matrix of ica result
%	FMRI.icaweight	weight matric of ica result
%       FMRI.icacomp    mark for meaningful components
%       FMRI.rejcomp    mark for rejected components
%       FMRI.ica_zthre  z threshold for ROA selection (default=1.5)
%       FMRI.ica_zbound upper bound for ROA display (default = 12.0);
%
%    - BOLD Image parameters:
%       FMRI.soa        soa of each trial
%       FMRI.trial_length length of each trial

% --
% Copyright (C) 2003 Jeng-Ren Duann, SCCN/INC/UCSD, duann@salk.edu
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2 of
% the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: fmrlab_chkdataset.m,v $


function fmrlab_chkdataset

global FMRI

if length(FMRI) == 0,
  fmrlab_messagedlg('fmrlab_chkdataset()',...
		    'Error:',...
		    ['No FMRI data available, please load dataset' ...
		     ' first']);
  return;
end

if isfield(FMRI,'data')==0,
  FMRI.data = [];
end
if isfield(FMRI,'filename')==0,
  FMRI.filename = [];
end
if isfield(FMRI,'pathname')==0,
  FMRI.pathname = [];
end
if isfield(FMRI,'dime_x')==0,
  FMRI.dime_x = [];
end
if isfield(FMRI,'dime_y')==0,
  FMRI.dime_y = [];
end
if isfield(FMRI,'dime_z')==0,
  FMRI.dime_z = [];
end
if isfield(FMRI,'fov_x')==0,
  FMRI.fov_x = [];
end
if isfield(FMRI,'fov_y')==0,
  FMRI.fov_y = [];
end
if isfield(FMRI,'slicetk')==0,
  FMRI.slicetk = [];
end
if isfield(FMRI,'dummy')==0,
  FMRI.dummy = [];
end
if isfield(FMRI,'timepoint')==0,
  FMRI.timepoint = [];
end
if isfield(FMRI,'tr')==0,
  FMRI.tr = [];
end
if isfield(FMRI,'idx')==0,
  FMRI.idx = [];
end
if isfield(FMRI,'processed')==0,
  FMRI.processed = '';
end
if isfield(FMRI,'struc_name')==0,
  FMRI.struc_name = '';
end
if isfield(FMRI,'struc_path')==0,
  FMRI.struc_path = '';
end
if isfield(FMRI,'struc_image_width')==0,
  FMRI.struc_image_width = [];
end
if isfield(FMRI,'struc_image_height')==0,
  FMRI.struc_image_height = [];
end
if isfield(FMRI,'struc_image_depth')==0,
  FMRI.struc_image_path = [];
end
if isfield(FMRI,'struc_fov_x')==0,
  FMRI.struc_fov_x = [];
end
if isfield(FMRI,'struc_fov_y')==0,
  FMRI.struc_fov_y = [];
end
if isfield(FMRI,'struc_slicetk')==0,
  FMRI.struc_slicetk = [];
end
if isfield(FMRI,'struc_resort')==0,
  FMRI.struc_resort = [];
end
if isfield(FMRI,'struc_flip')==0,
  FMRI.struc_flip = [];
end
if isfield(FMRI,'pcano')==0,
  FMRI.pcano = [];
end
if isfield(FMRI,'annealdeg')==0,
  FMRI.annealdeg = [];
end
if isfield(FMRI,'extended')==0,
  FMRI.extended = [];
end
if isfield(FMRI,'icasphere')==0,
  FMRI.icasphere = [];
end
if isfield(FMRI,'icaweight')==0,
  FMRI.icaweight = [];
end
if isfield(FMRI,'icacomp')==0,
  FMRI.icacomp = [];
end
if isfield(FMRI,'rejcomp')==0,
  FMRI.rejcomp = [];
end
if isfield(FMRI,'ica_zthre')==0,
  FMRI.ica_zthre = [];
end
if isfield(FMRI,'ica_zbound')==0,
  FMRI.ica_zbound = [];
end
if isfield(FMRI,'soa')==0,
  FMRI.soa = [];
end
if isfield(FMRI,'trial_length')==0,
  FMRI.trial_length = [];
end
if isfield(FMRI,'maxsteps')==0,
  FMRI.maxsteps = [];
end
if isfield(FMRI,'alignslice')==0,
  FMRI.alignslice = [];
end
