% makemovie() - create an .avi movie from Matlab .fig files or from a 3-D plot
%               viewed from the given view path (see 'path'). Stereographic 
%               movies may also be generated (see 'stereo'). Requires binary
%               movie-encoding program (mkavi).
% Usage:
%   >> makemovie( {filenamelist}, 'key', 'val', ...);
%   >> makemovie(  figure_handle, 'key', 'val', ...);
%
% Inputs:
%   filenamelist   - *.fig file name cell array giving .fig filename base 
%                    and [first last] indices { 'basefile' 0 100 }. A 4th
%                    parameter may optionally give the format of the number 
%                    in the filename: Ex. { 'frame' 0 100 4 } indicates that 
%                    the 101 .fig filenames are 'frameXXXX.fig', where XXXX 
%                    is 0000 to 0100. Here, 4 is the number of digits (XXXX). 
%                    Note: If the figures contain two copies of a 3-D plot,
%                    makemovie can generate stereo movies (see 'stereo' 
%                    option below).
% else
%   figure_handle  - Handle of a figure to be 3D-rotated to generate the 
%                    movie (see 'path' option below). If the figure has
%                    two copies of a 3-D plot, the 'stereo' option
%                    below may be used to produce a stereo movie.
%
% Optional input:
%   'outname'   - Output .avi file name, '.avi' is appended to the given file 
%                 name. Default is "out.avi".
%   'outsize'   - [width height] OR 'same'. Output movie width and height 
%                 (except when using the 'crop' option). If the string 'same'
%                 is given as argument, the original size of the figure is 
%                 used. Default is 'same'.
%   'shape'     - ['same'|'stretch'] Deform the image to fit the output size. 
%                 If 'same', add white flankers; if 'stretch', deform the image 
%                 to fit the output. Default is ?????.
%   'antialias' - [real magnification factor] Before creating a .ppm image, 
%                 antialias it by creating a larger image and applying a 
%                 smoothing filter. Factor=1 -> no antialiasing. Default is 3. 
%   'smooth'    - [integer > 0] Width of the gaussian window for smoothing
%                 the antialiased images. Default is 3.
%   'crop'      - [lo_x lo_y width height]. Crop the images before generating 
%                 the movie. [lo_x lo_y] coordinates of the lower left corner; 
%                 [width height] the width and height of the output image in
%                 pixels (Note: there is no pixel (0,0) in an image of size 
%                 'outsize' (above)). Default is no crop.
%   'mode'      - ['fast'|'getframe'|'slow'] Mode for generating frames
%                 for the movie: 'fast' only uses one pass; smoothing
%                 cannot be performed. 'getframe' uses the getframe() 
%                 function to capture the image. Its speed is OK, but
%                 each image must be shown on the screen and the size of the 
%                 antialiased image cannot be larger than the screen.
%                 The 'slow' mode writes intermediary files to disk, allowing
%                 large-factor antialiasing. Default is 'fast'.
%   'dir'       - Directory in which the images are located. Default is pwd().
%   'path'      - Array of size [3,n] frame view triples [theta phi frame_number]. 
%                 Here, theta is horiz./azimuth and phi the elevation angle as in view(). 
%                 Positive theta rotates counterclockwise (from nose); pos. elevation goes up.
%                 A figure handle must be the first function argument. If the bottom row
%                 contains Nan's, continue the path in the reverse of the given order 
%                 For example: Make a movie of 60 frames (30 specified + 30 return):
%                     [-135    0  1;    % Begin (frame 1) with a  right post. lateral view.
%                      -180  45 10;    % At frame 10, arrive at central posterior elevation.
%                       135  10 30;    % At frame 30, descend to left posterior elevation.
%                       Nan Nan Nan];  % During frames 31-60, reverse this path.
%                 Note: if the last viewpoint is the same as the first, this point 
%                 will not be duplicated to insure smooth periodic animation.
%   'stereo'    - [Real] Create a stereo movie. The figure should contain a [left right]
%                 display of two identical 3-D plots. The left plot view will follow the 
%                 given 'path' (see above). The right plot axis will be 3-D rotated by an 
%                 additional horizontal disparity angle specified by the 'stereo' argument:
%                 6 (degrees) suggested. Default is [] = mono display.
%   'deletetmp' - ['ask' | 'yes' | 'no'] specify whether the generated temporary .ppm 
%                 image files are to be deleted after the movie has been generated. 
%                 Default: 'ask', prompting the user. Use 'yes' or 'no' to run makemovie
%                 without user interaction (e.g., useful from scripts).
%
% Note: The movie file is generated in the directory where the images are located.
%
% Example:
%     % Make a movie from image files:
%     >> makemovie({'myframes' 1 30 3}, 'smooth', 3, 'outsize', [640 480]); 
%     % Make a movie of a rotating 3-D figure:
%     >> makemovie(gcf, 'outsize', [640 480], 'path',[-135 30 1; 0 45 100]);
%
% Author: Arnaud Delorme, CNL / Salk Institute, 9 July 2002
%
% See also: eeglab(), mkavi (binary movie-encoding routine)

%123456789012345678901234567890123456789012345678901234567890123456789012

% Copyright (C) 2002 Arnaud Delorme, Salk Institute, arno@salk.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% $Log: makemovie.m,v $
% Revision 1.25  2003/06/02 01:08:58  arno
% header
%
% Revision 1.24  2003/02/12 23:18:36  jorn
% added option 'deletetmp' to avoid user interaction
% fixed printf bug
% corrected help for outname
%
% Revision 1.23  2003/01/03 01:55:21  arno
% removing unix rm integer result
%
% Revision 1.22  2003/01/02 22:53:56  arno
% adding test when applying 3-D path
%
% Revision 1.21  2002/12/29 22:51:44  scott
% stereo suggested value
% ,
%
% Revision 1.20  2002/12/29 22:46:59  scott
% stereo bug
%
% Revision 1.19  2002/12/29 22:42:22  scott
% typo
%
% Revision 1.18  2002/12/29 22:40:01  scott
% debug rm
%
% Revision 1.17  2002/12/29 22:36:15  scott
% debug rm
%
% Revision 1.16  2002/12/29 22:32:52  scott
% typo
%
% Revision 1.15  2002/12/29 22:30:58  scott
% added rm query
%
% Revision 1.14  2002/12/29 22:14:11  scott
% made mkavi silent, added 'ls -l' of output .avi file. Changed .avi file default.
%
% Revision 1.13  2002/12/29 22:01:44  scott
% mkavi comments
%
% Revision 1.12  2002/12/29 21:24:37  arno
% debug path
%
% Revision 1.11  2002/12/29 00:33:34  scott
% header
%
% Revision 1.10  2002/12/28 02:02:16  scott
% header details
%
% Revision 1.9  2002/12/27 23:31:40  arno
% programming new path. Editing header to add an additional path feature (end of 'path' parameter description)
%
% Revision 1.8  2002/12/27 04:40:03  scott
% remade help message for 'path' and 'stereo' options -sm
%
% Revision 1.7  2002/12/27 03:33:09  arno
% adding 'path', ... parameter
%
% Revision 1.6  2002/11/19 18:35:09  arno
% constraining size
%
% Revision 1.5  2002/11/15 02:17:39  scott
% help msg
%
% Revision 1.4  2002/11/02 02:58:02  arno
% removing unix touch file
%
% Revision 1.3  2002/11/02 02:55:01  arno
% implementing cropping
%
% Revision 1.2  2002/11/02 01:40:12  arno
% imting slow mode
% and fast mode
%
% Revision 1.1  2002/11/01 18:23:25  arno
% Initial revision
%

function res = makemovie( filenamelist, varargin);
    
    if nargin < 1
        help makemovie;
        return;
    end;
    
    % checking input parameters
    % -------------------------
    g = finputcheck(varargin, { 'antialias'    'real'     [1 Inf]  1;
                                'smooth'       'real'     [1 Inf]  [];
                                'outname'      'string'   []       'out';
                                'deletetmp'    'string'   {'ask' 'yes' 'no'} 'ask';
                                'dir'          'string'   []       pwd;
                                'crop'         'integer'  [1 Inf]  [];
                                'mode'         'string'   {'fast' 'getframe' 'slow'}       'fast';
                                'multiproc'    'string'   { 'on' 'off' 'run' }  'off';
                                'shape'        'string'   { 'same' 'stretch' }  'same';
                                'stereo'       'real'     []                    [];
                                'path'         'real'     []                    [];
                                'outsize'      { 'integer' 'string' } []  'same' }, 'makemovie');
    if isstr(g), error(g); end;
    
    % more checking
    % -------------
    if ~isnumeric(filenamelist) & ~iscell(filenamelist)
        error('makemovie: first input must be a cell of length greater or equal to 3');
    end;
    if ( g.antialias~=1 | ~isempty(g.smooth) | ~isempty(g.crop)) & strcmpi(g.mode, 'fast')
        disp('makemovie: smooth, antialias & crop are incompatible with ''fast'' mode, switching to ''slow''');
        g.mode = 'slow';
    end;
    if ~isstr(g.outsize)
        if mod(g.outsize(1),4) ~= 0 | mod(g.outsize(2),4)
            error('Output width and height must be a multiple of 4');
        end;
    end;
    
    % decode input list
    % -----------------
    if iscell(filenamelist) & ~isstr(filenamelist{2})
        nzeros = 0;
        if length(filenamelist) == 4, nzeros = filenamelist{4}; end;
        newfilelist = {};
        count = 1;
        for index = filenamelist{2}:filenamelist{3}
            if nzeros, newfilelist{count} = sprintf(['%s%' int2str(nzeros) '.' int2str(nzeros) 'd'], ...
                                                    filenamelist{1}, index);
            else       newfilelist{count} = sprintf( '%s%d', filenamelist{1}, index);
            end;
            count = count + 1;
        end;
        if strcmp(g.outname, 'out'), g.outname = filenamelist{1}; end;
        filenamelist = newfilelist;
    end;
    
    % save old dir
    % ------------
    olddir = pwd;
    if ~isempty(g.dir)
        cd(g.dir);
    end;
    
    % 3-D path
    % --------
    if ~isempty(g.path)
        % test for NaNs
        % -------------
        if isnan(g.path(end,1))
            g.path(end,:) = [];
            middle = g.path(end,3);
            for index = size(g.path,1)-1:-1:1
                g.path(end+1,:) = g.path(index,:);
                g.path(end,3)   = middle+middle-g.path(index,3);
            end;
        end;
        
        % extrapolate path
        % ----------------
        count = 1;
        for index = 1:size(g.path,1)-1
            pathfact = linspace(0,1,g.path(index+1,3)-g.path(index,3)+1);
            newpath(count:count+length(pathfact)-1,:) = (1-pathfact') * g.path(index,:) + (pathfact')* g.path(index+1,:);
            count = size(newpath,1);
        end;
        g.path = newpath;
        if all(g.path(end,1:2) == g.path(end,1:2)), g.path(end,:) = []; end;
        g.path(:,3) = [];
    end;
    
    % running on the cluster
    % ----------------------
    if strcmpi(g.multiproc, 'on')
        listproc = {};
        movopt = { 'smooth' g.smooth 'outname' g.outname 'shape' g.shape 'outsize' g.outsize ...
                   'antialias' g.antialias 'multiproc' 'run' 'dir' g.dir };
        NPROC = 4;
        fileperproc = ceil(length(filenamelist) / NPROC);
        for index = 1:NPROC
            filelist =  filenamelist((index-1)*fileperproc+1:min(index*fileperproc, length(filenamelist)));
            listproc(index) = { { 'makemovie' { filelist movopt{:} } { 'res' } } };
        end;
        
        qsubfunc(listproc);
        
        count = zeros(1,length(filenamelist));
        fprintf('Scanning for output files ...\n'); 
        while sum(count) ~= length(filenamelist)
            
            % scan non-terminated files
            % -------------------------
            tmpindices = shuffle(find(count == 0));
            filename = sprintf('tmpimg%s.ppm', filenamelist{tmpindices(1)});
            if exist(filename) == 2
                fprintf('file %s found:', filename);
                tmpfile = dir(filename);
                if tmpfile.bytes == 0 % failure, have to relauch process
                    fprintf(' error, restarting process\n');
                    unix(['rm -f ' filename ]);
                    qsubfunc(listproc{ floor(tmpindices/NPROC)+1 } );
                else 
                    fprintf('OK\n');
                    count( tmpindices(1) ) = 1;
                end;
            end;
            pause(0.11);
        end;   
        
    else
        %try
            
        % loading and processing images
        if iscell(filenamelist)    nbitems = length(filenamelist);
        else                       nbitems = size(g.path, 1); fig = gcf;
        end;
        for index = 1:nbitems
            
            % loading file
            % ------------
            if iscell(filenamelist)
                % find real file name
                % -------------------
                tmpfilename = filenamelist{index};
                if exist(tmpfilename) ~= 2
                    if exist([tmpfilename '.fig']) ~= 2
                        if exist([tmpfilename '.FIG']) ~= 2
                            error(['makemovie: cannot find file ' tmpfilename '[.fig|.FIG]']);
                        else
                            tmpfilename = [tmpfilename '.FIG'];
                        end;
                    else
                        tmpfilename = [tmpfilename '.fig'];
                    end;
                end;
                
                % loading and resizing image
                % --------------------------
                fprintf('Processing image %s...\n', tmpfilename);
                tmpprop.visible = 'off';
                tmpprop.paperpositionmode = 'auto';
                fig = hgload(tmpfilename, tmpprop);
                tmpfilename = filenamelist{index}; 
            else 
                fprintf('Processing view number %d; view (%2.1f,%2.1f)...\n', index, g.path(index,1), g.path(index,2));
                tmpfilename = sprintf('%4.4d', index);
            end;
            
            % applying 3-D path
            % -----------------
            if ~isempty(g.path)
                tmph = get(fig, 'children');
                for newindex = 1:length(tmph)
                    try, set(tmph(newindex), 'view', g.path(index,:)); catch, end;
                end;
            end;
            
            % aplying stereo
            % --------------
            if ~isempty(g.stereo)
                tmph = get(fig, 'children');
                if length(tmph) ~= 2, error('Stereo error: current figure does not contain 2 plots'); end;
                tmppos  = get(tmph, 'position');
                [tmpview1 tmpview2] = view(tmph(1));
                if tmppos{1}(1) > tmppos{1}(2)
                     set(tmph(1), 'view', [tmpview1+g.stereo tmpview2]);
                else set(tmph(2), 'view', [tmpview1+g.stereo tmpview2]);
                end;
            end;
            
            pos = get(fig, 'position');
            width  = pos(3);
            heigth = pos(4);
            if isstr(g.outsize)
                g.outsize = [];
                g.outsize(1) = ceil(pos(3)/4)*4;
                g.outsize(2) = ceil(pos(4)/4)*4;
            end;
            factw  = g.outsize(1)*g.antialias/width;
            facth  = g.outsize(2)*g.antialias/heigth;
            
            if strcmp(g.mode, 'fast') % fast movie mode
                r = 0.8465;
                set(fig, 'position', [ 0 0 floor(g.outsize(1)*g.antialias/r), floor(g.outsize(2)*g.antialias/r) ]);
                com = sprintf('print -dppm -loose tmpimg%s.ppm', tmpfilename);
                eval(com);
                set(fig, 'position', [ 0 0 width heigth ]);
            else
                if strcmp(g.mode, 'getframe') % grab using getframe
                    if strcmpi(g.shape, 'same')
                        fact = min(factw, facth);
                        set(fig, 'position', [ 0 0 width*fact heigth*fact ]);
                    else
                        set(fig, 'position', [ 0 0 g.outsize(1)*g.antialias, g.outsize(2)*g.antialias ]);
                    end;
                    curframe = getframe(fig);
                    set(fig, 'visible', 'off');
                    curframe = curframe.cdata;
                else % slow movie mode (use tiff)
                     % ---------------
                    if strcmpi(g.shape, 'same')
                        % resize the image before getframe
                        % --------------------------------
                        fact = min(factw, facth);
                        com = sprintf('print -dtiff -loose -r%d tmpimg.tiff', floor(85*fact));
                        eval(com);
                    else
                        factratio = factw/facth;
                        heigth = heigth*factratio;
                        set(fig, 'position', [ pos(1) pos(2) width  heigth]);
                        com = sprintf('print -dtiffnocompression -loose -r%d tmpimg.tiff', floor(85*factw));
                        eval(com);
                    end;
                    curframe = double(imread('tmpimg.tiff'));
                end;
                
                % add flankers to the image
                % -------------------------
                [heigth width tmp] = size(curframe);
                if exist('newframe') == 0 | (size(newframe,1) ~= g.outsize(2)*g.antialias) | ...
                    (size(newframe,1) ~= g.outsize(1)*g.antialias)
                    newframe = ones(g.outsize(2)*g.antialias, g.outsize(1)*g.antialias, 3)*255;
                else 
                    newframe(:,:,:) = 255;
                end;
                diffh = g.outsize(2)*g.antialias - heigth;
                diffw = g.outsize(1)*g.antialias - width;
                newframe(round(diffh/2)+1:round(diffh/2)+heigth, ...
                         round(diffw/2)+1:round(diffw/2)+width,:) = curframe;
                
                % the image has been acquired, antialias it
                % -----------------------------------------
                if ~isempty(g.smooth)
                    
                    % building and apllying filter
                    % ----------------------------
                    %b = remez(10,[0 1/g.antialias-0.05 1/g.antialias+0.05 1],[0 0 1 1]);
                    %b = ftrans2(b);
                    %newframe (:,:, 1) = filter2( b, newframe (:,:, 1), 'same');
                    %newframe (:,:, 2) = filter2( b, newframe (:,:, 2), 'same');
                    %newframe (:,:, 3) = filter2( b, newframe (:,:, 3), 'same');
                    %tmpmin = min(newframe(:));
                    %newframe = newframe - tmpmin;
                    %tmpmax = max(newframe(:));
                    %newframe = newframe / (tmpmax/255);
                    
                    convgauss = gauss2d(g.smooth, g.smooth, g.smooth/3, g.smooth/3);
                    convgauss = convgauss / sum(convgauss(:));
                    newframe (:,:, 1) = conv2( newframe (:,:, 1), convgauss, 'same');
                    newframe (:,:, 2) = conv2( newframe (:,:, 2), convgauss, 'same');
                    newframe (:,:, 3) = conv2( newframe (:,:, 3), convgauss, 'same');
                end;
                if g.antialias ~= 1
                    % not necessary since imresize does it
                    newframe = imresize(newframe, [g.outsize(2) g.outsize(1)]);
                end;
                newframe = newframe / 255;
                %figure; image(newframe);
                
                % cropping image before saving ?
                % ------------------------------
                if ~isempty(g.crop)
                    newframe = newframe(g.crop(2):g.crop(2)+g.crop(4)-1, :);
                    newframe = newframe(:, g.crop(1):g.crop(1)+g.crop(3)-1);
                end;
                
                % write a ppm file with the image
                % -------------------------------
                pnmwrite(newframe, sprintf('tmpimg%s.ppm', tmpfilename));
            end;
            
            if iscell(filenamelist)
                close(fig);
            end;
        end;
        
        %catch, 
        %    disp(lasterr); return;
            %unix( sprintf('touch tmpimg%s%4.4d.ppm', filenamelist{1}(1), index) );
        %end;
    end;
    
    % Launch the binary executable program mkavi 
    % ----------------------------
    if ~strcmp(g.multiproc, 'run') & nbitems > 1
        fprintf('Making movie %s.avi file with mkavi (text output disabled)...', g.outname);
        unix(sprintf('mkavi -file %s.avi tmpimg*.ppm >& /dev/null', g.outname ));
    end;
    
    ;                    % Delete the temporary .ppm files, if desired
    cd(olddir);
    
    ;                    % action depends on input option 'deletetmp'
    switch g.deletetmp
     case 'ask'          % prompt user whether files are to be deleted
      unix(sprintf('echo ""; ls -l %s.avi; echo ""',g.outname));
      ques=sprintf('Remove %d tmpimg*ppm files (y/n)? ',nbitems);
      answ=input(ques,'s');
      if answ=='y'
          fprintf('Removing %d tmpimg*.ppm files ...\n',nbitems);
          unix('/bin/rm tmpimg*.ppm');
      else
          fprintf('Leaving %d tmpimg*.ppm files ...\n',nbitems);
      end
     case 'yes'          % delete ppm files
      unix('/bin/rm tmpimg*.ppm');
     case 'no'           % do nothing
     otherwise
      error('makemovie: wrong value for deletetmp!');
    end % switch
    
    ;                    % return
    res = 1;
    return;
