% map_on_struc() - component browser by overlaying ROA onto 2D
%                  slices of structural images with interactive 
%                  graphic user interface
%
% Usage:
%   >> map_on_struc(action)
%
% Inputs:
%   action - specifying action for the browse window to perform,
%            the value can be:
%          - 'initialize': initialize the graphic user interface
%          - 'prev_component': move to the previous coponent
%          - 'next_component': move to the next component
%          - 'goto_comp': goto the specified component indicated in
%            the text editing box
%          - 'add_list': add current component to the selection list
%          - 'goto_list': goto component highlighted in the
%            selection list
%          - 'remove_list': remove component from the selection list
%          - 'add_rej_list': add component into rejection list
%          - 'goto_rej_list': goto the component highlighted in the
%            rejection list
%          - 'rm_rej_list': remove component from the rejection list
%          - 'refresh': update the current view due to the change
%             of component selection or other actions
% 
% See also: map_on_fmri(), boldimage(), cbar()
%
% Notes: - map_on_struc() might need interpolation to make the ROA
%          the same coordinates with structural images. Thus, it might take
%          longer to browse component maps. It's highly recommended to use
%          map_on_fmri() first to add component of interest into the
%          selection list, then to use map_on_struc() to browse the selected
%          component only
% 
% --
% Copyright (C) 2002 Jeng-Ren Duann, Salk Institute, duann@salk.edu
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2 of
% the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

function map_on_struc(action)

global FMRI
global BASEIMG
global struc_image_width
global struc_image_height
global FS
global sf
global tw
global axg

if length(FMRI)==0,
    fmrlab_messagedlg('Component ROAs on Struct. Images',...
        'Error:',...
        ['No current data set exists. Please load a ''.fmr''' ...
            ' data file first']);
    return;
end

temp = findstr(FMRI.processed,'ica_trained');
if isempty(temp),
    fmrlab_messagedlg('map_on_struc()',...
        'Error:',...
        'No ICA results, please run ''Process >> Run ICA''');
    return;
end

if isempty(FMRI.struc_name),
    str = sprintf('No Structural images available.');
    str = [str sprintf(' See:\nDataset > Edit Struct. Info')];
    fmrlab_messagedlg('Component ROAs on Struct. Images',...
        'Error:',...
        str);
    return;
end

RGB_mainwin = [1 .984 .895];
RGB_background = [0.860 0.824 0.618];
RGB_button = [0.860 0.824 0.618];
RGB_text = [0.333 0.283 0.003];

pos_w = 700*sf(1); % window width
pos_h = 500*sf(2); % window height
axg = 5*min(sf); % gap in x- and y-direction
bw = 150*sf(1); % width of button
bh = 20*sf(2); % height of button
tw = 18; % width of title bar

if nargin<1,
    action = 'init';
end

switch(action),
case 'init',
    f0 = findobj('name','Display Component ROA on Structral Images');
    if ~isempty(f0),
        figure(f0);
    else
        disp('Create all controls.');
        S = get(0,'screensize');
        pos = [0 S(4)-tw-pos_h pos_w pos_h];
        f0 = figure( ...
            'name', 'Display Component ROA on Structural Images', ...
            'numbertitle','off',...
            'doublebuffer', 'on', ...
            'color',RGB_mainwin,...
            'menubar','none',...
            'position',pos,...
            'backingstore','off',...
            'deletefcn','map_on_struc3(''close'');');
        % determine row and column for display according to slice no.
        disp_col = ceil(sqrt(FMRI.dime_z));
        disp_row = ceil(FMRI.dime_z/disp_col);
        % determine the size of image object
        a = fix((pos_w-bw-(disp_col+1)*axg)/disp_col);
        b = fix((pos_h-(disp_row+1)*axg-bh)/disp_row);
        axw = min(a,b);
        % determine margins for image display
        disp_margin(1) = fix((pos_w-bw-(disp_col+1)*axg-disp_col*axw)/2);
        disp_margin(2) = fix((pos_h-bh-(disp_row+1)*axg-disp_row*axw)/2);
        % create axis for images
        im_ax = [];
        ind = 1;
        for y=1:disp_row,
            for x=1:disp_col,
                pos(1) = disp_margin(1) + (x-1)*(axg+axw);
                pos(2) = pos_h - disp_margin(2) - bh - y*(axg+axw);
                pos(3) = axw;
                pos(4) = axw;
                f1 = axes(...
                    'Parent',f0, ...
                    'units','pixels',...
                    'position',pos,...
                    'Color',RGB_mainwin, ...
                    'box','on',...
                    'Tag','image_ax',...
                    'xtick',[],...
                    'xticklabel',[],...
                    'ytick',[],...
                    'yticklabel',[]);
                im_ax = [im_ax f1];
                ind = ind+1;
                if ind > FMRI.dime_z,
                    break;
                end
            end
        end
        % create component title bar
        pos = [disp_margin(1)+axg pos_h-disp_margin(2)-bh-axg disp_col*axw+(disp_col-1)*axg bh];
        uicontrol(...
            'parent',f0,...
            'style','text',...
            'position',pos,...
            'fontsize',FS,...
            'fontweight','bold',...
            'string','Independent Component 1',...
            'tag','component_title',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text);
        % text for 'Browse'
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-bh-axg bw bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','text',...
            'string','Browse',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold');
        % button set for browsing
        pos = [pos_w-axg-bw/3 pos_h-disp_margin(2)-axg-2*bh bw/3 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','>',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_fore',...
            'callback','map_on_struc3(''foreward'');');
        pos = [pos_w-axg-2*bw/3 pos_h-disp_margin(2)-axg-2*bh bw/3 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','edit',...
            'string','1',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','ed_comp',...
            'callback','map_on_struc3(''refresh'');');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-axg-2*bh bw/3 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','<',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_back',...
            'callback','map_on_struc3(''backward'');');
        % text & buttons for 'Select Component'
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-3*bh-2*axg bw bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','text',...
            'string','Select Components',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold');
        pos = [pos_w-axg-bw/2 pos_h-disp_margin(2)-2*axg-4*bh bw/2 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','UnMark',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_unmarkselect',...
            'callback','map_on_struc3(''remove_select'');');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-2*axg-4*bh bw/2 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','Mark',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_markselect',...
            'callback','map_on_struc3(''add_select'');');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-2*axg-5*bh bw bh];
        if length(FMRI.icacomp) == 0,
            listStr = '0';
        else
            listStr = int2str(FMRI.icacomp);
            component_selected = FMRI.icacomp;
        end
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','popupmenu',...
            'string',listStr,...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','select_comp',...
            'value',1,...
            'callback','map_on_struc3(''goto_select'');');
        % text & buttons for 'Reject Component'
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-6*bh-3*axg bw bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','text',...
            'string','Reject Components',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold');
        pos = [pos_w-axg-bw/2 pos_h-disp_margin(2)-3*axg-7*bh bw/2 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','UnMark',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_unmarkreject',...
            'callback','map_on_struc3(''remove_reject'');');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-3*axg-7*bh bw/2 bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','pushbutton',...
            'string','Mark',...
            'backgroundcolor',RGB_button,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','bt_markreject',...
            'callback','map_on_struc3(''add_reject'');');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-3*axg-8*bh bw bh];
        if length(FMRI.rejcomp) == 0,
            rejstr = '0';
        else
            rejstr = int2str(FMRI.rejcomp);
            component_rejected = FMRI.rejcomp;
        end
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','popupmenu',...
            'string',rejstr,...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold',...
            'tag','reject_comp',...
            'value',1,...
            'callback','map_on_struc3(''goto_reject'');');
        % text and buttons for time course display
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-9*bh-4*axg bw bh];
        uicontrol(...
            'parent',f0,...
            'position',pos,...
            'style','text',...
            'string','Time Course(s)',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'fontsize',FS,...
            'fontweight','bold');
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-10*bh-4*axg bw bh];
        str = {'None', 'Comp.', 'ROA', 'Comp+ROA', 'BOLD Image'};
        check1 = uicontrol(...
            'parent',f0,...
            'style','popupmenu',...
            'tag','time_course',...
            'position',pos, ...
            'string',str, ...
            'fontsize',FS,...
            'fontweight','bold',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'value',1,...
            'callback','map_on_struc3(''tc_clicked'');');
        % Other operations
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-11*bh-6*axg bw bh];
        str = {'Other Operations','PVAF vs Z thre','ER_BOLD',...
                'Close Comp Browser'};
        uicontrol(...
            'parent',f0,...
            'style','popupmenu',...
            'tag','other_operation',...
            'position',pos, ...
            'string',str, ...
            'fontsize',FS,...
            'fontweight','bold',...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'value',1);
        % create control for colorbar
        %cb_h = .7*(pos_h-12*bh-6*axg);
        %pos = [pos_w-axg-.85*bw pos_h-12*bh-6*axg-cb_h 30*sf(1) cb_h];
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-13*bh-7*axg bw bh];
        h1 = axes(...
            'Parent',f0, ...
            'units','pixels',...
            'position',pos,...
            'Color',RGB_mainwin,...
            'box','on',...
            'Tag','cb_ax',...
            'xtick',[],...
            'xticklabel',[],...
            'ytick',[],...
            'yticklabel',[]);
        % plot colorbar
        cb = linspace(65,128,64);
        cb = cb';
        b = repmat(cb,1,5);
        kk = image(b');
        set(h1,'ytick',[]);
        set(h1,'xtick',linspace(1,64,10));
        cb = linspace(-8.0,-1.5,5);
        cb = [cb linspace(1.5,8.0,5)];
        cb = num2str(cb','%4.1f');
        i = [2:4 7:9];
        for j=1:size(cb,2),
            cb(i,j) = ' ';
        end
        set(h1,'xticklabel',cb, 'fontsize',FS-2);
        fmrlab_color(2);
        cb_ax = h1;
        % create edit for z_limit
        %S = get(cb_ax,'position');
        %pos = [S(1)+S(3)+axg S(2)+S(4)-bh bw/2 bh];
        pos = [pos_w-axg-bw/2 pos_h-disp_margin(2)-15*bh-7*axg bw/2 bh];
        h1 = uicontrol(...
            'parent',f0,...
            'style','edit',...
            'string','8.0',...
            'position',pos,...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'tag','ed_lim',...
            'fontsize',FS,...
            'callback','map_on_struc3(''refresh'');',...
            'fontweight','bold');
        % create edit for z-threshold
        %pos = [S(1)+S(3)+axg S(2)+S(4)-2*bh-axg bw/2 bh];
        pos = [pos_w-axg-bw pos_h-disp_margin(2)-15*bh-7*axg bw/2 bh];
        h1 = uicontrol(...
            'style','edit',...
            'string','1.5',...
            'position',pos,...
            'backgroundcolor',RGB_mainwin,...
            'foregroundcolor',RGB_text,...
            'tag','ed_thre',...
            'fontsize',FS,...
            'callback','map_on_struc3(''refresh'');',...
            'fontweight','bold');
        disp('Done.');
        
        % display the first component
        disp('Calculating inverse weight matrix and component maps, please wait...');
        W = FMRI.icaweight * FMRI.icasphere;
        invW = pinv(W);
        u = W * FMRI.data;
        disp('Done.');
        % save useful information for further use
        userdata.im_ax = im_ax;
        userdata.cb_ax = cb_ax;
        userdata.u = u;
        userdata.invW = invW;
        set(f0,'userdata',userdata);    
        
        display_component_map;
    end

case 'close',
    a = findobj('tag','select_comp');
    b = str2num(get(a,'string'));
    if b(1) ~= 0,
        FMRI.icacomp = b;
    else
        FMRI.icacomp = [];
    end
    a = findobj('tag','reject_comp');
    b = str2num(get(a,'string'));
    if b(1) ~= 0,
        FMRI.rejcomp = b;
    else
        FMRI.rejcomp = [];
    end
    f_main = findobj('name','Display Component ROA on Structural Images');
    f_time = findobj('name','Component Time Courses');
    if ~isempty(f_time),
      delete(f_time);
    end
    f_bold = findobj('name','BOLD Image Plot');
    if ~isempty(f_bold)
      delete(f_bold);
    end
    if ~isempty(f_main),
      delete(f_main);
    end
    
case 'foreward'
    a = findobj('tag','ed_comp');
    b = str2num(get(a,'string'));
    if b == size(FMRI.icaweight,1),
        b = 1;
    else
        b = b+1;
    end
    set(a,'string',num2str(b));
    display_component_map;

case 'backward'
    a = findobj('tag','ed_comp');
    b = str2num(get(a,'string'));
    if b==1,
        b = size(FMRI.icaweight,1);
    else
        b = b-1;
    end
    set(a,'string',num2str(b));
    display_component_map;
    
case 'refresh',
    display_component_map;
    
case 'goto_select',
    a = findobj('tag','select_comp');
    b = get(a,'string');
    c = get(a,'value');
    c = str2num(b(c,:));
    a = findobj('tag','ed_comp');
    set(a,'string',num2str(c));
    display_component_map;
    
case 'add_select',
    a = findobj('tag','ed_comp');
    b = str2num(get(a,'string'));
    a = findobj('tag','select_comp');
    c = get(a,'string');
    if str2num(c(1,:)) == 0,
        set(a,'string',num2str(b));
	    FMRI.icacomp = b;
    else
        d = str2num(c);
        found = 0;
        for ii=1:length(d),
            if b == d(ii);
                found = 1;
                break;
            end
        end
        if found == 1,
            fmrlab_messagedlg('map_on_fmri3()',...
                'Warning:',...
                'Item in selecting list already');
        else
            d = [d;b];
	        FMRI.icacomp = sort(d(:));
            set(a,'string',num2str(sort(d)));
        end
    end

case 'remove_select',
    % if no items in the list, warning
    a = findobj('tag','select_comp');
    b = get(a,'string');
    if str2num(b(1,:)) == 0,
        fmrlab_messagedlg('map_on_struc3()',...
            'Error:',...
            'No items in the list!');
        return;
    end
    % if the select item doesn't match the current displayed 
    % component, warning
    c = str2num(b);
    d = get(a,'value');
    aa = findobj('tag','ed_comp');
    bb = str2num(get(aa,'string'));
    if bb ~= c(d);
        fmrlab_messagedlg('map_on_struc3()',...
            'Error:',...
            'Please select the component you want to delete from the Select List first');
        return;
    end
    c(d) = [];
    FMRI.icacomp = c(:);
    if isempty(c),
        set(a,'string','0');
    else
        set(a,'string',num2str(c));
        set(a,'value',1);
    end
    
case 'goto_reject',
    a = findobj('tag','reject_comp');
    b = get(a,'string');
    c = get(a,'value');
    c = str2num(b(c,:));
    a = findobj('tag','ed_comp');
    set(a,'string',num2str(c));
    display_component_map;
    
case 'add_reject',
    a = findobj('tag','ed_comp');
    b = str2num(get(a,'string'));
    a = findobj('tag','reject_comp');
    c = get(a,'string');
    if str2num(c(1,:)) == 0,
        set(a,'string',num2str(b));
	    FMRI.rejcomp = b;
    else
        d = str2num(c);
        found = 0;
        for ii=1:length(d),
            if b == d(ii);
                found = 1;
                break;
            end
        end
        if found == 1,
            fmrlab_messagedlg('map_on_fmri3()',...
                'Warning:',...
                'Item in rejecting list already');
        else
            d = [d;b];
	        FMRI.rejcomp = sort(d(:));
            set(a,'string',num2str(sort(d)));
        end
    end
    
case 'remove_reject',
    % if no items in the list, warning
    a = findobj('tag','reject_comp');
    b = get(a,'string');
    if str2num(b(1,:)) == 0,
        fmrlab_messagedlg('map_on_struc3()',...
            'Error:',...
            'No items in the list!');
        return;
    end
    % if the select item doesn't match the current displayed 
    % component, warning
    c = str2num(b);
    d = get(a,'value');
    aa = findobj('tag','ed_comp');
    bb = str2num(get(aa,'string'));
    if bb ~= c(d);
        fmrlab_messagedlg('map_on_struc3()',...
            'Error:',...
            'Please select the component you want to delete from the Reject List first');
        return;
    end
    c(d) = [];
    FMRI.rejcomp = c(:);
    if isempty(c),
        set(a,'string','0');
    else
        set(a,'string',num2str(c));
        set(a,'value',1);
    end
    
case 'tc_clicked'
	a = findobj('tag','time_course');
	b = get(a,'value');
	if b == 1,
		f = findobj('name','Component Time Courses');
		if ~isempty(f),
			close(f);
		end
		f = findobj('name','BOLD Image Plot');
		if ~isempty(f),
			close(f);
		end
	else
		display_component_map;
	end
    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function display_component_map

global FMRI
global tw
global axg
global FS
global sf

RGB_mainwin = [1 .984 .895];
RGB_background = [0.860 0.824 0.618];
RGB_button = [0.860 0.824 0.618];
RGB_text = [0.333 0.283 0.003];

f_main = findobj('name',['Display Component ROA on Structural' ...
    ' Images']);
if f_main == 0,
    disp('Cannot find main window');
    return;
end
userdata = get(f_main,'userdata');
% form base image
fname = [FMRI.struc_path FMRI.struc_name];
fp = fopen(fname,'rb','ieee-be');
avg_image = fread(fp,'uint16');
fclose(fp);
struc_image_width = FMRI.struc_image_width;
struc_image_height = FMRI.struc_image_height;
struc_slice_no = FMRI.struc_image_depth;
base_image = zeros(1,struc_image_width*struc_image_height*struc_slice_no);
base_image = fmrlab_scale(avg_image,1,64);
BASEIMG = base_image;
base_image = reshape(base_image,struc_image_width, ...
    struc_image_height,struc_slice_no);
% find the displaying parameters
a = findobj(f_main,'tag','ed_thre');
uthre = str2num(get(a,'string'));
lthre = -1 * uthre;
a = findobj(f_main,'tag','ed_lim');
ulim = str2num(get(a,'string'));
llim = -1 * ulim;
% update ticks of colorbar
cb = linspace(llim,lthre,5);
cb = [cb linspace(uthre,ulim,5)];
cb = num2str(cb','%4.1f');
i = [2:4 7:9];
for j=1:size(cb,2),
    cb(i,j) = ' ';
end
set(userdata.cb_ax,'xticklabel',cb);
% get current component
a = findobj(f_main,'tag','ed_comp');
comp = str2num(get(a,'string'));
% calculating component map
temp_data = userdata.u(comp, :);
mean_temp_data = mean(temp_data);
std_temp_data = std(temp_data);
temp_data = (temp_data-mean_temp_data)/std_temp_data; %convert to z score
uidx = find(temp_data >= uthre);
temp_data1 = zeros(FMRI.dime_x*FMRI.dime_y*...
    FMRI.dime_z,1);
for i=1:length(FMRI.idx),
    temp_data1(FMRI.idx(i)) = temp_data(i);
end
temp_data1 = reshape(temp_data1,FMRI.dime_x, FMRI.dime_y, ...
    FMRI.dime_z);
if struc_image_width ~= FMRI.dime_x,
    xs = linspace(1, struc_image_width, FMRI.dime_x);
    ys = linspace(1, struc_image_height, FMRI.dime_y);    
    [xi, yi] = meshgrid(1:struc_image_width, 1:struc_image_height); 
    smap = [];
    for i=1:FMRI.dime_z,
        temp_data2 = interp2(xs, ys, temp_data1(:,:,i), xi, yi);
        temp_data2 = reshape(temp_data2, struc_image_width*struc_image_height, 1);
        smap = [smap temp_data2];
    end
else
    smap = temp_data1;
end
smap = reshape(smap,struc_image_width*struc_image_height*...
    FMRI.dime_z,1);
pos_smap = smap;
ii = find(pos_smap > ulim);
pos_smap(ii) = ulim;
ii = find(pos_smap < uthre);
pos_smap(ii) = 0.0;
ii = find(pos_smap >= uthre);
pos_smap(ii) = fmrlab_scale(pos_smap(ii),99,128);
BASEIMG = reshape(BASEIMG,struc_image_width*struc_image_height*FMRI.dime_z,1);
img = max(BASEIMG,pos_smap);
neg_smap = smap;
ii = find(neg_smap < llim);
neg_smap(ii) = llim;
ii = find(neg_smap > lthre);
neg_smap(ii) = 0.0;
ii = find(neg_smap <= lthre);
neg_smap(ii) = fmrlab_scale(neg_smap(ii),65,94);
img = max(img,neg_smap);
img = reshape(img,struc_image_width,struc_image_height,FMRI.dime_z);

% display component maps
for ii=1:FMRI.dime_z,
    axes(userdata.im_ax(ii));
    image(img(:,:,ii)');
    axis off
    axis image
end
fmrlab_color(2);

% update the title bar according to the current component
a = findobj(f_main,'tag','component_title');
set(a,'string',['Independent Component ' num2str(comp)]);

% display time couse and bold image plots if specified
a = findobj(f_main,'tag','time_course');
b = get(a,'value');
% display time course if b ~= 1
if b > 1,
    % calculate back-projected component time course and
    % + ROA mean time course
    ddd = zeros(size(userdata.invW,1),size(userdata.u,2));
    ddd = userdata.invW(:,comp) * userdata.u(comp,:);
    ddd = ddd(:,uidx)';
    bpctc = mean(ddd);
    ddd = FMRI.data(:,uidx);
    ddd = mean(ddd');
    bpctc = 100 * (bpctc-mean(bpctc)) / (mean(ddd));
    roatc = 100 * (ddd-mean(ddd))/(mean(ddd));

    f_time = findobj('name','Component Time Courses');
    if isempty(f_time),
        S1 = get(0,'screensize');
        S2 = get(f_main,'position');
        pos = [S2(1) 0 S2(3) S1(4)-S2(4)-2*tw-axg];
        f_time = [];
        f_time = figure(...
            'position',pos,...
            'numbertitle','off',...
            'menubar','none',...
            'color',RGB_mainwin,...
            'name','Component Time Courses');
    else
        figure(f_time);
        clf;
    end
    
    switch(b)
    case 2,
        if isempty(FMRI.tr) | FMRI.tr == 0,
            plot(bpctc);
            set(gca,'fontsize',FS);
            xlabel('Scan Number');
        else
            x1 = linspace(FMRI.tr,FMRI.timepoint*FMRI.tr,FMRI.timepoint);
            plot(x1,bpctc);
            set(gca,'fontsize',FS);
            xlabel('Time (s)');
        end
        pos = get(gca,'position');
        set(gca,'position',[pos(1) pos(2)+pos(4)*0.1 pos(3) pos(4)*0.9]);
        legend('Comp. proj. to +ROA');
        ylabel('\Delta BOLD Signal (%)');
        set(gca,'ylim',[min(bpctc) max(bpctc)]);
        set(gca,'ytick',[min(bpctc) 0 max(bpctc)]);
        
    case 3,
        if isempty(FMRI.tr) | FMRI.tr == 0,
            plot(roatc,'r');
            set(gca,'fontsize',FS);
            xlabel('Scan Number');
        else
            x1 = linspace(FMRI.tr,FMRI.timepoint*FMRI.tr,FMRI.timepoint);
            plot(x1,roatc,'r');
            set(gca,'fontsize',FS);
            xlabel('Time (s)');
        end
        pos = get(gca,'position');
        set(gca,'position',[pos(1) pos(2)+pos(4)*0.1 pos(3) pos(4)*0.9]);
        legend('+ROA mean time course');
        ylabel('\Delta BOLD Signal (%)');
        set(gca,'ylim',[min(roatc) max(roatc)]);
        set(gca,'ytick',[min(roatc) 0 max(roatc)]);
            
    case 4,
        if isempty(FMRI.tr) | FMRI.tr == 0,
            plot(bpctc);
	        hold on
	        plot(roatc,'r');
	        set(gca,'fontsize',FS);
	        xlabel('Scan Number');
        else
            x1 = linspace(FMRI.tr,FMRI.timepoint*FMRI.tr, ...
                FMRI.timepoint);
	        plot(x1,bpctc);
	        hold on
	        plot(x1,roatc,'r');
	        set(gca,'fontsize',FS);
	        xlabel('Time (s)');
        end
        pos = get(gca,'position');
        set(gca,'position',[pos(1) pos(2)+pos(4)*0.1 pos(3) pos(4)*0.9]);
        legend('Comp. proj. to +ROA','+ROA mean time course');
        ylabel('\Delta BOLD Signal (%)');
        set(gca,'ylim',[min(min(roatc),min(bpctc))...
		    max(max(roatc),max(bpctc))]);
        set(gca,'ytick',[min(min(roatc),min(bpctc))...
		    0 max(max(roatc),max(bpctc))]);
        v = 100*((1-var(roatc-bpctc)/var(roatc)));
        %ti = textsc(0.85,0.05,[sprintf('p.v.a.f. = %5.2f',v) '%']);
        ti = title([sprintf('p.v.a.f. = %5.2f',v) '%']);
        set(ti,'fontsize',FS,'color',[1 0 1]);
    
    case 5,
        if isempty(FMRI.tr) | FMRI.tr == 0,
            plot(bpctc);
            set(gca,'fontsize',FS);
            xlabel('Scan Number');
        else
            x1 = linspace(FMRI.tr,FMRI.timepoint*FMRI.tr,FMRI.timepoint);
	        plot(x1,bpctc);
	        set(gca,'fontsize',FS);
	        xlabel('Time (s)');
        end
        pos = get(gca,'position');
        set(gca,'position',[pos(1) pos(2)+pos(4)*0.1 pos(3) pos(4)*0.9]);
        legend('Comp. proj. to +ROA');
        ylabel('\Delta BOLD Signal (%)');
        set(gca,'ylim',[min(bpctc) max(bpctc)]);
        set(gca,'ytick',[min(bpctc) 0 max(bpctc)]);
       
        % find bold image plot window
        f_bold = findobj('name','BOLD Image Plot');
        if isempty(f_bold),
            S1 = get(0,'screensize');
            S2 = get(f_time,'position');
            pos = [S2(1)+S2(3) 0 S1(3)-S2(3)-2*axg 2*S2(4)];
            f_bold = [];
            f_bold = figure(...
                'position',pos,...
                'numbertitle','off',...
                'menubar','none',...
                'color',RGB_mainwin,...
                'name','BOLD Image Plot');
        else
            figure(f_bold);
            clf;
        end
       
        % enter FMRI.soa if missing
        if length(FMRI.soa) == 0,
            prompt = {'Trial onsets (scan numbers):',...
                    'Trial scans (in scans):'};
            defs = {num2str(FMRI.soa), num2str(FMRI.trial_length)};
            BOLD_err = 0;
            a = fmrlab_inputdlg(prompt,'BOLD-image plot parameters',1,defs);
	        if ~isempty(a),
                FMRI.soa = str2num(char(a(1)));
                if length(FMRI.soa)==1 | length(FMRI.soa)==0,
                    fmrlab_messagedlg('Component ROAs on fMRI',...
                        'Error:',...
                        'Too few trials');
                    BOLD_err = 1;
                end
   
	            for i=1:length(FMRI.soa),
                    if FMRI.soa(i) > FMRI.timepoint | FMRI.soa(i) < 1,
                        fmrlab_messagedlg('Component ROAs on fMRI',...
                            'Error:',...
                            'Onset scan number exceeds total scan number');
                        BOLD_err = 1;
                        break;
                    end
                end	   
                FMRI.trial_length = str2num(char(a(2)));
                if FMRI.trial_length == 0,
                    fmrlab_messagedlg('Component ROAs on fMRI',...
                        'Error:',...
                        'Trial length cannot be zero');
                    BOLD_err = 1;
                end
                
                for i=1:length(FMRI.soa),
                    temp = FMRI.trial_length + FMRI.soa(i) - 1;
                    if temp > FMRI.timepoint,
                        fmrlab_messagedlg('Component ROAs on fMRI',...
                            'Error:',...
                            'Trial length exceeds the total scan number');
                        BOLD_err = 1;
                        break;
                    end
                end
            else      
                BOLD_err = 1;
            end
            
            if BOLD_err == 1,
                a = findobj(f_main,'tag','list_tc');
                set(a,'value',1);
                close(f_time);
                close(f_bold);
                return;
            end
        end
        
        a = []; 
        for i=1:length(FMRI.soa),
            a = [a bpctc(FMRI.soa(i):FMRI.soa(i)+FMRI.trial_length-1)];
        end
        boldimage(a,ones(length(FMRI.soa),1),...
            0:FMRI.tr:FMRI.tr*(FMRI.trial_length-1),...
            ['Component ' num2str(comp)],1,1,'erp','nosort','noplot','cbar');
    end
end
